  #include <admodel.h>
  #include <df1b2fun.h>
  #include <adrndeff.h>
  const int JMAX=50;
  typedef df1b2variable (df1b2_parameters::*DFPMF) (const df1b2variable&);
  void function_minimizer::trapzd(DFPMF,const double a,
    const double b,const df1b2variable& _s,int ns=9);

  class df1b2_parameters;

  void function_minimizer::trapzd(
    df1b2variable (df1b2_parameters::*func)(const df1b2variable&),
    const double a,
    const double b, const df1b2variable& _ss,int n)
  {
    ADUNCONST(df1b2variable,ss)

    double x,num_interval,hn;
    df1b2variable sum;
    static df1b2variable s;
    static int interval;
    int j;

    df1b2_parameters * ptr= (df1b2_parameters *) mycast();
      (ptr->*func)(s);
    if (n == 1) {
      interval=1;
      s=0.5*(b-a)*
        ((ptr->*func)(df1b2variable(a))+(ptr->*func)(df1b2variable(b)));
      ss=s;
    } else {
      num_interval=interval;
      hn=(b-a)/num_interval;
      x=a+0.5*hn;
      for (sum=0.0,j=1;j<=interval;j++,x+=hn) sum += (ptr->*func)(x);
      interval *= 2;
      s=0.5*(s+(b-a)*sum/num_interval);
      ss=s;
    }
  }

  void function_minimizer::adromb(
     df1b2variable (df1b2_parameters::*func)(const df1b2variable&),
     double a,double b,const df1b2variable& _v,int ns)
  {
    const double base = 4;
    int MAXN = min(JMAX, ns);
    df1b2vector s(1,MAXN+1);
  
    for(int j=1; j<=MAXN+1; j++)
    {
      trapzd(func,a,b,s[j],j);
    }
  
    for(int iter=1; iter<=MAXN+1; iter++)
    {
      for(int j=1; j<=MAXN+1-iter; j++)
      {
        s[j] = (pow(base,iter)*s[j+1]-s[j])/(pow(base,iter)-1);
      }
    }
    return s[1];
  }

#include <admodel.h>

#include <df1b2fun.h>

#include <adrndeff.h>

  extern "C"  {
    void ad_boundf(int i);
  }
#include <s.htp>

  df1b2_parameters * df1b2_parameters::df1b2_parameters_ptr=0;
  model_parameters * model_parameters::model_parameters_ptr=0;
model_data::model_data(int argc,char * argv[]) : ad_comm(argc,argv)
{
  n.allocate("n");
  width.allocate("width");
  xs.allocate(1,n,"xs");
}

model_parameters::model_parameters(int sz,int argc,char * argv[]) : 
 model_data(argc,argv) , function_minimizer(sz)
{
  model_parameters_ptr=this;
  initializationfunction();
  beta.allocate("beta");
  sigeps.allocate(0.000001,5,"sigeps");
  u.allocate(1,n,"u");
  f.allocate("f");  /* ADOBJECTIVEFUNCTION */
  sigma.allocate("sigma");
  #ifndef NO_AD_INITIALIZE
  sigma.initialize();
  #endif
}
void model_parameters::userfunction(void)
{
   int j;
   f=0;
   for (j=1;j<=n;j++)
   {
      ll_j(j,beta,sigeps,u(j));
   }
}

void SEPFUN1  model_parameters::ll_j(const int j, const dvariable& beta,const dvariable& sigeps,const dvariable& u)
{
  begin_df1b2_funnel();
   dvariable eps=u*sigeps;
   int k;
   dvariable mu;
   dvariable sigma=exp(beta+eps);
   mu=adromb(&model_parameters::fct,0,width,8);
   f -= -0.5*square(u);
   f -= -log(mu) - 0.5*square(xs(j)/sigma);
  end_df1b2_funnel();
}

dvariable model_parameters::fct(const dvariable& x)
{
   dvariable tmp;
   tmp=mfexp(-.5*x*x/square(sigma));
   return tmp;
}
  long int arrmblsize=0;

int main(int argc,char * argv[])
{
  ad_set_new_handler();
  ad_exit=&ad_boundf;
    gradient_structure::set_NO_DERIVATIVES();
    gradient_structure::set_YES_SAVE_VARIABLES_VALUES();
  #if defined(__GNUDOS__) || defined(DOS386) || defined(__DPMI32__)  || \
     defined(__MSVC32__)
      if (!arrmblsize) arrmblsize=150000;
  #else
      if (!arrmblsize) arrmblsize=25000;
  #endif
    df1b2variable::noallocate=1;
    df1b2_parameters mp(arrmblsize,argc,argv);
    mp.iprint=10;

    function_minimizer::random_effects_flag=1;
    df1b2variable::noallocate=0;
    mp.preliminary_calculations();
    initial_df1b2params::separable_flag=1;
    mp.computations(argc,argv);
    return 0;
}

extern "C"  {
  void ad_boundf(int i)
  {
    /* so we can stop here */
    exit(i);
  }
}

void model_parameters::preliminary_calculations(void){
  admaster_slave_variable_interface(*this);
  }

model_data::~model_data()
{}

model_parameters::~model_parameters()
{}

void model_parameters::report(void){}

void model_parameters::final_calcs(void){}

void model_parameters::set_runtime(void){}

#ifdef _BORLANDC_
  extern unsigned _stklen=10000U;
#endif


#ifdef __ZTC__
  extern unsigned int _stack=10000U;
#endif


#define __DF1B2PART__
void df1b2_parameters::user_function(void)
{
   int j;
   f=0;
   for (j=1;j<=n;j++)
   {
      ll_j(j,beta,sigeps,u(j));
   }
}

void   df1b2_pre_parameters::ll_j(const int j, const funnel_init_df1b2variable& beta,const funnel_init_df1b2variable& sigeps,const funnel_init_df1b2variable& u)
{
  begin_df1b2_funnel();
   df1b2variable eps=u*sigeps;
   int k;
   df1b2variable mu;
   df1b2variable sigma=exp(beta+eps);
   adromb(&df1b2_parameters::fct,0,width,mu,8);
   f -= -0.5*square(u);
   f -= -log(mu) - 0.5*square(xs(j)/sigma);
  end_df1b2_funnel();
}

df1b2variable df1b2_parameters::fct(const df1b2variable& x)
{
   df1b2variable tmp;
   tmp=mfexp(-.5*x*x/square(sigma));
   return tmp;
}
   
void df1b2_pre_parameters::setup_quadprior_calcs(void) 
{ 
  df1b2_gradlist::set_no_derivatives(); 
  quadratic_prior::in_qp_calculations=1; 
}  
  
void df1b2_pre_parameters::begin_df1b2_funnel(void) 
{ 
  (*re_objective_function_value::pobjfun)=0; 
  other_separable_stuff_begin(); 
  f1b2gradlist->reset();  
  if (!quadratic_prior::in_qp_calculations) 
  { 
    df1b2_gradlist::set_yes_derivatives();  
  } 
  funnel_init_var::allocate_all();  
}  
 
void df1b2_pre_parameters::end_df1b2_funnel(void) 
{  
  lapprox->do_separable_stuff(); 
  other_separable_stuff_end(); 
} 
  
void model_parameters::begin_df1b2_funnel(void) 
{ 
  if (lapprox)  
  {  
    {  
      begin_funnel_stuff();  
    }  
  }  
}  
 
void model_parameters::end_df1b2_funnel(void) 
{  
  if (lapprox)  
  {  
    end_df1b2_funnel_stuff();  
  }  
} 

void df1b2_parameters::allocate(void) 
{
  beta.allocate("beta");
  sigeps.allocate(0.000001,5,"sigeps");
  u.allocate(1,n,"u");
  f.allocate("f");  /* ADOBJECTIVEFUNCTION */
  sigma.allocate("sigma");
  #ifndef NO_AD_INITIALIZE
  sigma.initialize();
  #endif
}
